# Gestion de la Base de de données

## Sécurité et contrôle d'accès

### Configuration du serveur

Il y a deux fichiers qui vont vous permettre de verouiller l'accés à votre base, le `postgresql.conf` et le `pg_hba.conf`.

Le postgresql.conf permet de jouer sur 

- SSL (Secure Sockets Layer) : Vous pouvez activer et configurer SSL pour sécuriser les connexions entre le client et le serveur PostgreSQL. Cela permet de chiffrer les données transitant entre les deux et d'empêcher les interceptions non autorisées. Vous pouvez spécifier les certificats SSL, les clés privées, les méthodes d'authentification et les protocoles SSL à utiliser.

- Authentification des utilisateurs : PostgreSQL prend en charge diverses méthodes d'authentification pour les utilisateurs. Vous pouvez configurer des méthodes d'authentification telles que le chiffrement des mots de passe (**password_encryption**), l'authentification par certificat client (ssl_cert_file, ssl_key_file), l'authentification GSSAPI (krb_server_keyfile), l'authentification PAM (pam configuration), etc. Vous pouvez choisir la méthode d'authentification appropriée en fonction de vos besoins de sécurité.

- Règles d'accès : Bien que les règles d'accès soient généralement configurées dans le fichier pg_hba.conf, certaines options de configuration de sécurité sont disponibles dans postgresql.conf. Par exemple, vous pouvez définir l'option **listen_addresses** pour spécifier les adresses IP sur lesquelles le serveur PostgreSQL écoute les connexions entrantes. Cela permet de restreindre l'accès au serveur en spécifiant les adresses IP autorisées.

- Limitation des ressources : PostgreSQL vous permet de définir des limites sur les ressources utilisées par les sessions et les requêtes. Vous pouvez configurer les paramètres tels que max_connections pour limiter le nombre de connexions simultanées, max_locks_per_transaction pour limiter le nombre de verrous pris par transaction, max_prepared_transactions pour limiter le nombre de transactions préparées, etc. Cela aide à prévenir les attaques de déni de service et à garantir une utilisation équitable des ressources du système.

- Journalisation (logging) : La configuration de la journalisation dans postgresql.conf vous permet de spécifier le niveau de détail des journaux, les fichiers de destination et les formats de sortie. Vous pouvez choisir de journaliser les requêtes, les erreurs, les avertissements, les connexions, etc. La journalisation adéquate est essentielle pour l'audit, la détection des problèmes de sécurité et la conformité aux réglementations.

Le fichier pg_hba.conf va permettre de faire une gestion plus fines des accés à la base de donnée.

```shell
# TYPE  DATABASE        USER            ADDRESS                 METHOD

# Autoriser toutes les connexions locales en tant qu'utilisateur postgres avec une méthode d'authentification par mot de passe
local   all             postgres                                md5
# Autoriser les connexions provenant de l'adresse IP spécifique avec une méthode d'authentification par mot de passe
host    all             all             192.168.1.100/32        md5
# Autoriser les connexion sur une base spécifique depuis une ip précise par mot de passe
host    vitis           all             192.168.1.100/32        md5
# Spécification d'une configuration pour forcer l'utilisation du SSL
hostssl vitis           all             127.0.0.1/32            md5
# Autoriser les connexions provenant de l'adresse IP spécifique avec une méthode d'authentification par mot de passe chiffré
host    mydatabase      myuser          10.0.0.5/24             scram-sha-256
# Autoriser une méthode de connexion spécifique uniquement pour un rolegroup spécifique
host    mydatabse       +mon_role_group 127.0.0.1/32            md5
```

A savoir que Postgres va tester chaque règle de maniére itérative tel que définir dans le fichier de haut en bas.
Il faut donc que la règle la plus restrictive se trouve avant les règle moins restrictive dans ce fichier.

Autres points : 

- Sauvegardes : Mettez en place une stratégie de sauvegarde régulière pour protéger vos données. Configurez des sauvegardes automatiques, en utilisant des outils tels que pg_dump ou des solutions de sauvegarde tierces pour garantir la disponibilité des données en cas de défaillance du système.

### Configuration de la base  

La gestion des droits peut se réveler trés complexe dans vos bases postgres. A savoir que des drtois bien géré sur une base de données est surement le meilleurs moyen de résister aux différentes attaques au niveau de l'application.

Pour se faire il faut respecter [le principe du moindre privilèges](https://fr.wikipedia.org/wiki/Principe_de_moindre_privil%C3%A8ge) qui s'applique habituellement à la gestion des OS mais qui dans le contexte d'une base de données Postgres est tout aussi valable.

La base de donnée sera la première couche du développement de vos applications et une gestion de droit bien faite dès cette première couche, fera que la gestion au niveau de vos autres couches découlera de manière naturelle. Autrement dit vous casser la tête sur ce point à ce moment là c'est vous éviter des failles et vous simplifier le travail pour la suite.

![webapp stack dev](./images/webapp-stack-dev.png)

Comme vous le savez tous, les droits sur la base sont géré via les rolegroupes postgres (ou privilèges Vitis), privilèges que vous retrouvez pour gérer la sécurité sur l'API ou gérer l'affichage au niveau du client.

#### Spécifités des vues et des règles

Dans une base de données PostgreSQL, lorsque vous utilisez une vue, vous n'avez pas besoin d'avoir directement les droits sur la table sous-jacente à la vue pour accéder aux données via cette vue. Cela est dû à la façon dont les vues sont gérées par PostgreSQL.

Une vue est une représentation virtuelle d'une ou plusieurs tables dans la base de données. Elle agit comme une couche d'abstraction qui masque la structure et les détails des tables sous-jacentes, offrant une interface simplifiée pour les utilisateurs. Lorsque vous créez une vue, vous pouvez spécifier les colonnes que vous souhaitez inclure et appliquer des conditions ou des jointures pour filtrer les données.

Lorsqu'un utilisateur accède à une vue, PostgreSQL vérifie les droits de l'utilisateur sur la vue elle-même, et non sur les tables sous-jacentes. Cela signifie que si vous avez les droits nécessaires pour accéder à la vue, vous pourrez récupérer les données de la vue, même si vous n'avez pas directement les droits sur les tables concernées.

Cependant, il est important de noter que les droits d'accès à la vue doivent être configurés correctement pour garantir la sécurité des données. Si un utilisateur a accès à une vue qui renvoie des données à partir d'une table, mais n'a pas les droits sur cette table, il ne pourra pas effectuer d'opérations de modification sur la vue. Les modifications doivent être effectuées directement sur les tables sous-jacentes, en respectant les droits d'accès appropriés.

Lorsqu'il s'agit d'effectuer des opérations d'insertion, de mise à jour (update) ou de suppression (delete) dans une base de données PostgreSQL, il est important de prendre en compte les règles et les contraintes définies pour maintenir l'intégrité et la cohérence des données. Voici une explication des règles générales :

Insertion (INSERT) : Lors de l'insertion de nouvelles données dans une table, les règles et les contraintes s'appliquent pour garantir que les données ajoutées sont conformes aux spécifications du schéma de la base de données. Cela comprend la vérification des clés primaires, des contraintes d'unicité, des contraintes de référence, etc. Si une violation de règle ou de contrainte se produit lors de l'insertion, l'opération sera rejetée et une erreur sera renvoyée.

Mise à jour (UPDATE) : Lors de la mise à jour des données existantes dans une table, les règles et les contraintes veillent à ce que les modifications apportées maintiennent l'intégrité des données. Par exemple, les contraintes de clé étrangère peuvent vérifier si les modifications respectent les références entre tables. Si une mise à jour viole une règle ou une contrainte, l'opération sera rejetée et une erreur sera renvoyée.

Suppression (DELETE) : Lors de la suppression de données d'une table, les règles et les contraintes garantissent que les opérations de suppression sont cohérentes avec les dépendances et les références dans la base de données. Par exemple, si une ligne est supprimée d'une table référencée par une clé étrangère dans une autre table, les contraintes de référence peuvent spécifier l'action à prendre, telle que la suppression en cascade des enregistrements correspondants dans la table référencée. Si une suppression viole une règle ou une contrainte, l'opération sera rejetée et une erreur sera renvoyée.

En plus des règles et des contraintes, les opérations d'insertion, de mise à jour et de suppression peuvent également être soumises à des vérifications de sécurité, telles que les permissions d'accès des utilisateurs et les rôles définis dans la base de données. Les utilisateurs doivent disposer des autorisations appropriées pour exécuter ces opérations sur les tables concernées.

Autre intéret à connaitre si une ligne n'est pas récupérable via le SELECT d'une vue pour un utilisateur, alors cet utilisateur ne pourra pas réaliser de modification/suppression sur ces lignes via des les règles de cette vue. Ce qui est un moyen simple d'améliorer la sécurité de vos applications.

Pour résumer en passant par les vues vous n'avez à gérer les droits que sur les vue, l'utilisateur n'a pas besoin d'accéder aux tables, en revanche si une séquence est utilisé pour une valeur par défaut, alors il faut que l'utilisateur puisse y accéder.
C'est une spécificité du SGBD de postgres.


## Sauvegarde et restauration des données

Il existe trois outils spécifiques à postgres 

- [pg_dump](https://docs.postgresql.fr/15/app-pgdump.html) : permet de créer un fichier de backup contenant la structure et/ou les données d'une base de donnée ou d'un de ses sous-objets.
- [pg_restore](https://docs.postgresql.fr/15/app-pgrestore.html) : permet de restaurer une backup réalisée avec pg_dump ou pg_dumpall
- [pg_dumpall](https://docs.postgresql.fr/15/app-pg-dumpall.html) : permet de créer un fichier contenant toutes les infos d'une base de donnée ou d'un cluster y compris les éléments partagés par tout le cluster.

## Maintenance et optimisation des performances

### Utilisation des vue matérialisées

Une vue matérialisée fonctionne comme une vue, à la différence que le résultat de la requete SELECT va être stocké, ce qui permet un accés à la donnée bien plus rapide vu que toutes les étapes de construction du retour de la requête ne sera pas à refaire.

La mise à jour de se retour est à demander explicitement via une requête SQL.

```sql
CREATE TABLE Customers (
    customer_id SERIAL PRIMARY KEY,
    customer_name VARCHAR(100)
);

CREATE TABLE Orders (
    order_id SERIAL PRIMARY KEY,
    order_date DATE,
    customer_id INTEGER REFERENCES Customers(customer_id)
);

INSERT INTO Customers (customer_name) VALUES ('John Doe');
INSERT INTO Customers (customer_name) VALUES ('Jane Smith');
INSERT INTO Customers (customer_name) VALUES ('Robert Johnson');

INSERT INTO Orders (order_date, customer_id) VALUES ('2022-01-01', 1);
INSERT INTO Orders (order_date, customer_id) VALUES ('2022-02-01', 1);
INSERT INTO Orders (order_date, customer_id) VALUES ('2022-03-01', 2);
INSERT INTO Orders (order_date, customer_id) VALUES ('2022-04-01', 3);

CREATE MATERIALIZED VIEW OrderCounts AS
SELECT c.customer_id, c.customer_name, COUNT(o.order_id) AS order_count
FROM Customers c
LEFT JOIN Orders o ON c.customer_id = o.customer_id
GROUP BY c.customer_id, c.customer_name;

REFRESH MATERIALIZED VIEW OrderCounts;

SELECT customer_name, order_count
FROM OrderCounts;
```

Danc ce cas là le count n'aura un impact de performance que lors de la mise à jour de la vue matérialisée, la requête qui sera faite sur la vue matérialisée répondra aussi vite que si on interrogait une simple table.

Cependant cette mise à jour statique présente un défaut, en cas d'utilisation d'une clause where, limitant le retour de données en fonction de l'utilisateur (comme la restrcition communal de vmap), la vue matérialisée contiendra les données de l'utilisateur ayant lancé le REFRESH.
Autrement dit dans ce cas là, il vaut mieux lancer une vue matérialisée, sans utilisé de condition ou uniquement ce qui ne va pas dépendre du role qui interroge la donnée ou des circonstances d'interrogation, en revanche il est pertinent de continuer à utiliser ces restriction lors de l'interrogation de la vue matérialisée.

Si la donnée change énormément et régulièrement, la vue matérialisée deviendra un handicape pour votre application.


### Importance des index

Il faut savoir que les colonnes soumis à des contraintes de type clés primaire ou unicité sont automatiquement indéxés avec un index de type B-TREE.

Il est préférable d'indexer les colonnes qui seront régulièrement soumis à des utilisation dans des jointure ou des clause where, cependant attention une surutilisation des index fera grossir le l'espace disque occupé par votre base et pourra avoir un effet néfaste sur l'utilisation de votre base de donnée voir même ralentir vos requêtes.

### Utilisation du VACUUM et automatisation

Dans PostgreSQL, la commande VACUUM est utilisée pour récupérer l'espace disque occupé par les tables et les index. Elle permet également de mettre à jour les statistiques internes utilisées par l'optimiseur de requêtes pour prendre des décisions efficaces lors de l'exécution des requêtes.

Voici ce que fait la commande VACUUM :

- Récupération de l'espace disque : Lorsqu'une transaction est effectuée dans PostgreSQL, les données modifiées sont marquées comme "supprimées" mais ne sont pas immédiatement supprimées physiquement de l'espace disque. Au fil du temps, ces données supprimées s'accumulent et peuvent entraîner une fragmentation de l'espace disque. Le VACUUM récupère cet espace disque en supprimant physiquement les données marquées comme supprimées.

- Mise à jour des statistiques : PostgreSQL utilise des statistiques internes sur les tables et les index pour prendre des décisions d'optimisation lors de l'exécution des requêtes. Ces statistiques incluent le nombre de lignes, la distribution des valeurs, etc. Le VACUUM met à jour ces statistiques pour refléter les modifications apportées aux données depuis la dernière exécution du VACUUM.

Il existe différentes variantes de la commande VACUUM qui peuvent être utilisées en fonction des besoins spécifiques :

- VACUUM : Cette commande effectue une récupération de l'espace et met à jour les statistiques pour toutes les tables et les index de la base de données.

- VACUUM FULL : Cette commande effectue une récupération de l'espace plus agressive en réorganisant physiquement les données et les index. Elle peut nécessiter davantage de temps et de ressources, mais elle peut récupérer plus d'espace disque.

- VACUUM ANALYZE : Cette commande effectue à la fois la récupération de l'espace et la mise à jour des statistiques. Elle est souvent utilisée pour maintenir les performances optimales de la base de données.

Il est recommandé d'exécuter régulièrement la commande VACUUM sur votre base de données pour maintenir l'intégrité des données, prévenir la fragmentation de l'espace disque et garantir des performances de requête optimales.

Il convient de noter que dans les versions plus récentes de PostgreSQL, le VACUUM est automatiquement planifié et exécuté en arrière-plan pour maintenir la base de données en bon état. Cependant, il est toujours recommandé de connaître et de comprendre l'utilisation de la commande VACUUM pour pouvoir l'exécuter manuellement si nécessaire.

### Utilisation d'EXPLAIN et ANALYZE

Dans PostgreSQL, les commandes EXPLAIN et ANALYZE sont utilisées pour analyser et optimiser les requêtes. Elles fournissent des informations détaillées sur la façon dont le moteur de base de données exécute une requête et permettent d'identifier les éventuels goulots d'étranglement ou problèmes de performance.

La commande EXPLAIN permet d'obtenir le plan d'exécution d'une requête sans exécuter réellement la requête. Elle affiche des informations telles que les types d'opérations effectuées, l'ordre d'exécution des opérations, l'utilisation des indexes, les jointures, etc. Voici un exemple d'utilisation de la commande EXPLAIN :

```sql
EXPLAIN SELECT * FROM Customers WHERE age > 30;
```

La commande ANALYZE, quant à elle, est utilisée pour collecter et mettre à jour les statistiques sur les tables et les indexes. Ces statistiques sont utilisées par l'optimiseur de requêtes pour évaluer le coût des différentes stratégies d'exécution et choisir le plan d'exécution le plus efficace. Voici un exemple d'utilisation de la commande ANALYZE :

```sql
ANALYZE Customers;
```

Lorsqu'elles sont utilisées ensemble, les commandes EXPLAIN ANALYZE exécutent réellement la requête en plus de fournir le plan d'exécution et les statistiques détaillées. Cela permet d'obtenir des informations précises sur le temps d'exécution de la requête, le nombre de lignes traitées, les temps d'accès aux données, etc. Voici un exemple d'utilisation de la commande EXPLAIN ANALYZE :

```sql
EXPLAIN ANALYZE SELECT * FROM Customers WHERE age > 30;
```

![Explain Analyze](./images/explain_analyze_result.png)

L'analyse des résultats de EXPLAIN ANALYZE permet d'identifier les éventuels problèmes de performance dans une requête et de prendre des mesures pour les résoudre. Par exemple, vous pouvez ajuster les indexes, réorganiser les jointures, ajouter des clauses WHERE supplémentaires, etc.

En résumé, l'utilisation des commandes EXPLAIN et ANALYZE dans PostgreSQL est essentielle pour comprendre et optimiser les performances des requêtes. Elles fournissent des informations précieuses sur le plan d'exécution et les statistiques des requêtes, ce qui permet d'identifier les problèmes de performance et de prendre des mesures pour les résoudre.