# Usages basiques

## Définir une clé SSH

L'ajout d'une clé SSH est préférable pour simplifier l'utilisation de Git depuis votre poste.
### Création ou utilisation d'une clé SSH existante pour GitKraken

Pour interagir avec vos clés SSH via GitKraken il faudra accéder au paramétrage en cliquant sur cette roue crantée.

![SettingGK](/images/git/gk_settings.png)

Il faudra ensuite aller dans le menu `SSH`

![SettingGK](/images/git/gk_ssh.png)

Les boutons **Browse** vous permettront de trouver une clé existante et de l'associer à GitKraken. \
Le bouton **Generate** permettra de créer une nouvelle paire de clés SSH.

Le bouton à droite de la clé publique vous permettra de copier le contenu de la clé publique pour l'étape suivante sur Gitlab.

### Ajout de la clé SSH dans Gitlab

Une fois connecté à l'interface Web, accéder à **vos préférences** via le menu en haut à droite : 

![Profile Gitlab](/images/git/gitlab_profile_menu.png)

Dans l'interface des préférences, il y a un menu dédié aux clés SSH : 

![Preference Gitlab](/images/git/gitlab_preference_menu.png)

Dans le formulaire il faut copier la clé publique uniquement dans le champ prévue pour ça, complétez le formulaire et finaliser l'ajout de la clé.

```{Tip}
  Vous pouvez avoir plusieurs clés SSH, si vous le souhaitez. \
  Si vous utilisez des clés SSH que vous utilisez également en dehors de Veremes ou sur des EC2, soyez vigilant pour qu'elle ne fuite pas et si ça devait être le cas, pensez à la dissocier de gitlab.
```

## Cloner un dépôt

Pour cloner un dépôt, il faut commencer par récupérer l'URL du dépôt sur Gitlab.

![Gitlab clone btn](/images/git/gitlab_clone_btn.png)

Comme on le voit sur le screenshot, il y a deux types d'URL, une pour utilisation via SSH et une via HTTPS.

L'URL SSH nécessite d'avoir configuré une clé SSH, par contre cela facilite énormément les interactions avec le dépôt distant.
L'URL HTTPS n'a pas de prérequis, par contre il faut saisir le login/password de Gitlab à chaque interaction avec le serveur distant.

Une fois que vous avez cet URL vous pouvez soit lancer la commande `git clone [URL] [Dossier_dest]` par défaut le dossier de destination sera le nom du dépôt.

Ou avec GitKraken,  `nouvel onglet > Clone a Repo`.

![Gk clone modale](/images/git/gk_clone_modal.png)

Choisisser un dossier contenant votre dépôt, coller l'URL que vous souhaitez utiliser, vous pouvez changer le nom du dossier du dépôt, puis cliquer sur clone. À ce moment-là un bandeau apparaîtra sur le haut de l'application pour savoir si vous souhaitez ouvrir le dépôt après l'avoir cloné.

```{Tip}
  Je vous conseille de créer un dossier Git sur votre système de fichiers et de tout stocker dedans.

  ![Git Suggestion Arbo](/images/git/git_sug_arbo.png)

  Vous pourrez dire à GitKraken de surveiller ce dossier, il pourra alors vous lister tous les dépôts Git à l'intérieur de ce dossier, et vous permettre de les ouvrir rapidement : dans la modale pour ouvrir un dépôt, il faut cliquer sur le bouton vert et choisir le dossier, dans la partie de droites vous aurez alors la liste de tous les dépôts dans ce dossier.

  ![Git Suggestion Arbo](/images/git/gk_clone_main_dir.png)

```

## Workflow Git

Avant de rentrer dans le fonctionnement du workflow et l'usage qu'on en fait petit rappel :
Une application Web Veremes a son propre dépôt qui fait référence à des dépôts Git de modules tous contenus dans le dossier src (a minima vitis et un autre module que ce soit gtf, vmap ou autres et potentiellement d'autres modules). Ce qui veut dire qu'une application Web Veremes c'est a minima trois dépôts Git.

### Principe

Veremes utilise un gitflow assez classique en y incorporant des éléments du gitlabflow.
La documentation sur le flow d'origine est disponible à [ce lien](https://docs.gitlab.com/ee/topics/gitlab_flow.html).

Par convention Veremes, utilise la branche **master** pour livrer les releases de ses applications, on peut considérer que c'est équivalent à une branche de production et la branche **next_version** pour les développements de nouvelles fonctionnalités.
Tous les projets sur Gitlab ont les labels **bug** et **évolutions**

Les versions qui sont représentées avec des tags au niveau du dépôt git sont toujours sous cette forme Année.Release.Correctif (ex : 2023.01.01, premier correctif de la première release de l'année 2023).

Avant de sortir une version c'est-à-dire de réaliser les tags, il est préférable de créer une branche de pré-release (pre-20XX.XX.XX) pour tester la génération du setup, et la version elle-même.

### Contexte d'application

![Flow sur la durée](/images/git/veremes_gitflow.svg)

La branche **master** doit toujours être stable et prête à produire un correctif.
En cas de remontée de bug on crée une **issue** sur gitlab en lui ajoutant le label **bug**, le maintainer/responsable du projet, viendra assigner cette issue à un milestone en fonction de l'urgence de la demande, voir il y assignera un développeur.

Le développeur devra alors reporter l'issue sur le ou les bon(s) dépôt(s), créer une branche et une merge request correspondantes sur chaque dépôt concerné. Un bouton permet de le faire depuis l'issue, il faut laisser le nom par défaut et adapter la source, pour un bug ce sera master, pour un développement sur next_version.

![Flow issue to mr](/images/git/gitflow_issue_mr.png)

Le développeur va ensuite récupérer la branche sur son poste, corriger le problème et envoyer les modifications vers le serveur. Il devra alors passer la merge request en prête (en retirant le **Draft** au début) et l'assigner à un maintainer du dépôt.

Le maintainer vérifiera alors que le correctif est fonctionnel et jugera de la qualité du code. En fonction de quoi, il renverra des informations au développeur pour réaliser des modifications ou validera le code. En validant la merge request le maintainer réalisera le merge de la branche du correctif vers master et supprimera la branche du correctif (donc uniquement la référence vers le commit HEAD de cette branche, pas les commit en eux-mêmes).

Il est également préférable de merger **master** dans **next_version** de façon à impacter le correctif sur la branche de développement également et s'éviter des risques de conflits ultérieurs.

En cas de développement de nouvelles fonctionnalités, c'est exactement la même chose, sauf que le label sur l'issue sera **évolutions** et que la branche source à utiliser au moment de la création de la merge request sera **next_version** (sauf exception cf. vitis pour vmap, ...).

Une fois le milestone terminé, que ce soit pour un correctif ou des évolutions, le maintainer devra suivre le processus de sortie de version classique (détaillé ailleurs dans cette documentation).

## Gestion de conflit

Régulièrement lors des merges vous allez rencontrer des conflits.

![GitKraken Arbre conflit](/images/git/gk_conflict_tree.png)

Il faudra alors les gérer de façon à ne pas perdre de fonctionnalités.

Gitkraken vous permet de le gérer assez facilement :

![GitKraken Manager conflit](/images/git/gk_conflict_manager.png)

Vous pourrez voir via ce composant, quels fichiers sont en conflit et lesquels ne le sont pas. En cliquant sur un fichier en conflit, vous arriverez sur une interface vous permettant de gérer le conflit sur ce fichier.

![GitKraken Manager conflit](/images/git/gk_conflict_manager_file.png)

L'interface vous présente alors les deux versions du fichier, vous pouvez choisir le code à conserver, que ce soit le fichier entièrement, le bloc de code ou à la ligne. Vous pouvez conserver les deux versions des modifications simultanément (l'ordre de sélection sera l'ordre final dans le fichier).
Si le merge nécessite de modifier le fichier, vous pouvez soit le faire dans un éditeur externe, soit le gérer dans la troisième version du fichier (sur la partie basse du composant gitKraken), en éditant le texte directement. Une fois le conflit géré sur le fichier il vous suffit de cliquer sur le bouton `save` pour résoudre le conflit sur ce fichier.

Une fois tous les fichiers gérés, vous pourrez finaliser votre commit de merge. Si vous n'êtes pas en capacité de merger, vous pouvez annuler le merge en cliquant sur `abort`. Il vous suffit alors de demander à l'autre développeur en cause dans le conflit, ou a quelqu'un avec plus d'expertise que vous sur le sujet de vous aider à résoudre le conflit.

```{Warning}
  Un conflit se manifeste dans le fichier par l'apparition de balise, en cas de mauvaise gestion du conflit (plus exactement en cas d'absence de gestion), les balises resteront et lèveront des erreurs de syntaxe.

  ![conflict file](/images/git/code_conflict.png)
```


## Utilisation du gitignore

Le fichier .gitignore permet de spécifier à git qu'il ne doit pas surveiller (track) certains fichiers, ça peut être un fichier en particulier, une extension de fichier, un dossier, ...

le .gitignore peut être placé à plusieurs niveaux dans l'arborescence, même s'il est conseillé de n'en avoir qu'un à la racine du dépôt.

L'écriture de ce fichier n'est pas très compliquée et GitKraken vous permet de le gérer facilement en faisant un clique droit dans votre zone de travail.

![GitKraken ignore](/images/git/gk_ignore.png)

```{Warning}
  Si vous ajoutez un fichier dans le .gitignore et que ce fichier est déjà présent sur le dépôt Git distant et que d'autres utilisateurs l'ont déjà récupéré. Pour que le fichier ne soit plus surveillé il faudra l'ajouter au .gitignore et commit l'effacement du fichier, en faisant ça le fichier sera effacé chez tous les développeurs au moment du pull. c'est pourquoi, il est fortement déconseillé d'ignorer un fichier qui est déjà sur le serveur distant, d'autant plus si sa présence reste nécessaire au fonctionnement de l'application.
```

## Commit un dossier vide

Git ne peut pas créer d'arbre s'il n'a pas de blob à associer. Il ne peut donc pas créer un commit ne comportant que la création d'un dossier.

Cependant, il existe bien un contournement à ce problème, il suffit de créer dans le dossier un fichier vide qui s'appellera par convention `.git_keep`.

Vous pourrez alors commit et push le dossier.

```{Warning}
  Lorsque le dossier n'est plus vide, c'est à la charge du développeur de supprimer le fichier vide qui devient alors inutile.
```

## Comparer les différences entre deux commits avec GitKraken

En maintenant la touche Ctrl enfoncée, il est possible de sélectionner plusieurs commits dans GitKraken. 
En faisant ça sur deux commits, il set possible de voir les différences entre ces deux commits.

![GitKraken diff](/images/git/gk_diff.png)

Par exemple au moment de sortir une version, vous pouvez comparer master ou votre branche de pré-release avec le tag précédent pour voir si vous devez générer un engine ou non. \
Autre exemple au moment d'assigner votre merge request à un maintainer, vous pouvez comparer le HEAD de votre branche avec le HEAD de la branche d'origine, pour vérifier que vous n'envoyez que du code utile et qu'il ne reste pas de trace de tests.

## Processus de sortie de version d'application

Il faut sortir une version d'une application suite à la finalisation d'un milestone, ou d'une commande et après avoir déjà réalisé des tests sur l'application.

En ce qui concerne la protection des dépôts, seul master et next_version ont un traitement particulier et uniquement en ce qui concerne le push. \
Pour toutes les autres fonctionnalités de Git, tout le monde peut tout faire. Il est bien sûr évident que la sortie d'une version se fait en accord avec le responsable du produit ou de la commande, mais vous êtes en capacité de réaliser toutes les étapes ci-dessous, ou de créer des merge-request pour qu'un maintainer puisse vous aider.

1. Création d'une branche de pre-release dont le nom suivra la règle habituelle `pre-20XX.XX.XX`, cette branche pourra venir de next_version si on sort une nouvelle_version ou master s'il s'agit d'un correctif urgent. Dans le cas où il s'agit d'une release d'évolution, il est important de merge également master dedans de façon à récupérer les correctifs qui auraient pu être fait et non ramené dans next_version entre-temps. Étape à réaliser sur l'application mais aussi sur les modules.
1. Modifier le `install_/dependencies.json` de l'application de façon à obtenir les bonnes dépendances (sur vos branches de pré-release ou des tags)
1. Si des modifications ont eu lieu sur les engines d'un des modules, générer l'engine avec Jenkins et retoucher le `dependencies.json` du module pour modifier la ressource à utiliser.
1. Générer un setup avec Jenkins, tester, ... éventuellement faite des correctifs sur la branche de pré-release.
1. Lorsque tout est bon, on peut alors ramener la branche de pre-release dans master et créer le tag de la version finale. Pour l'application le tag peut être fait directement sur la branche de pre-release, en fonction de la quantité de modifications sur cette dernière (si vous avez changé que les versions dans le fichier json, c'est cohérent de tag directement sur la branche de pré-release).
1. Générer le setup finale en mettant à jour le dépendencies avec les tags et le publier si necessaire.
1. Merge master dans next_version pour intégrer d'éventuels correctifs dans les futures évolutions. Éventuellement suite à quoi il faudra remettre en état les dependencies.json des applications sur master/next_version pour garder une cohérence.

```{Note}
  Le processus vu ici est le processus dans le meilleur des cas, il peut y avoir des variations. \
  L'important c'est d'assimiler les grands principes et la logique du Gitflow qui est derrière.
```


## Processus de documentation

La documentation est générée automatiquement via un **CI/CD GitLab** en utilisant **Sphinx**.

On considère que la documentation sur la branche **master** est la documentation à mettre à disposition pour les clients, qui part sur le bucket **documentation.veremes.net**. \
La documentation sur **next_version** part sur un bucket de tests/préproduction afin de tester la compilation avec Sphinx, à savoir **documentation-dev.veremes.net** (ce bucket n'est accessible que depuis l'adresse IP de Veremes).

```{Note}
  Exception faite de cette documentation qui n'est disponible que sur le bucket de développement.
```

Il peut arriver que la documentation soit gérée sur une branche **documentation** pour une écriture plus intensive en début de projet. \
Les issues dédiées à la rédaction de documentation sont habituellement associées au label **documentation**.

L'écriture de la documentation suit le flow habituel en temps normal.

La documentation est validée avec le responsable produit et le service Marketing si possible.

En ce qui concerne la rédaction des documentations des applications, une charte est disponible à [ce lien](http://documentation.veremes.net/charte/index.html).