# Outils et fonctionnement

## Git

[Git](https://git-scm.com/) est un outil de versionnement, comme SVN/Subversion (sauf que c'est vachement mieux 😎).
c'est à dire qu'il permet de suivre les évolutions faites sur nos fichier de codes (et autres), et éventuellement de revenir en arrière facilement.

Les grande force de Git sont, l'utilisation des branches, sa gestion des modifications et son architecture, qui en font un outil incontournable.

```{Note}
  Git a été conçu par Linus Torvalds, qui est un des concepteurs du noyau Linux, il a notamment développé une bonne partie du système de fichier qu'utilise encore Linux aujourd'hui. Ce qui est interressant c'est que Git fonctionne sensiblement comme un système de fichier.
```

### Installation

Il faut passer par le site [Git scm](https://git-scm.com/downloads), tout est décrit sur le site.

Pour linux un simple `apt install git` fera l'affaire.

### Fondamentaux

#### Architecture

Architecture classique d'un outil comme SVN : 

![Architecture classique](/images/git/svn-archi.png)

Architecture Git classique : 

![Architecture Git](/images/git/git-archi.png)

Ce qui veut dire que chaque développeur à une copie locale du dépôt distant, en cas de perte ou indisponibilité du dépôt c'est un avantage significatif.

Un dépôt Git local peut être lié à plusieurs dépôts distants (cas que nous ne verront pas).


#### Git, comment ça marche ?

Git ne sait gérer que trois objets : 

- **Les blobs** : représente le contenu d'un fichier ou des modifications partielles sur un fichier, ainsi qu'une signature (hash SHA1) de ce contenu.
- **Les arbres** : liste de pointeur sur des blobs et/ou d'autres arbres (récupère également les droits Linux sur les fichiers), cette liste donne aussi suite à une signature SHA1
- **Les commits** : Référence sur un arbre et a minima un autre commit (exception faite du commit initial), comporte aussi un message (commentaire) et un SHA1, référence un auteur et un commiter (les deux pouvant être différents)

```{Note}
  Le seul objet qui est clairement lisible pour vous au final c'est le commit.
```

```{Tip}
  Le commentaire sur le commit se divise en deux, la première ligne qui sera facilement visible dans l'arbre, et une deuxième non visible par défaut mais qui vous permet de donner beaucoup plus d'informations.
```


Au final un dépôt git c'est une arborescence de commits rien de plus.

Pour en savoir plus sur le fonctionnement en profondeur de Git, je vous conseille [les chapitres 10.1, 10.2 et 10.3 de la documentation sur Git Scm](https://git-scm.com/book/fr/v2/Les-tripes-de-Git-Plomberie-et-porcelaine).

Ces objets vont être gérés à travers plusieurs zones distinctes.

1. **Zone de travail (Working area)** : C'est votre dossier de travail tel que vous le voyez basiquement, et vous ne manipulerez manuellement que cette zone.
1. **Zone de cache/préparation (Staging area)** : Permet en réalité de générer les objets blobs et arbres avant de les associer à un commit
1. **Dépôt local (dossier .git)** : votre arborescence de commit personnelle (toutes les branches sont présentes)
1. **Dépôt distant** : L'arborescence de commit mutualisée, qui est le plus souvent unique et que l'on appelle par convention **origin**.

![HEAD 1](/images/git/git_area.svg)

L'état de votre Zone de travail sera relatif à un commit, c'est-à-dire que les différences sont calculées relativement à un commit existant. 
Ce commit se nomme le HEAD (c'est le commit courant de votre zone de travail) : 

![HEAD 1](/images/git/detached_head.png)

#### Les autres "objets" Git

Quand on parle de Git, on parle systématiquement de branches, de merges et souvent de tags. Hors comme je l'ai dit précédemment en réalité ces objets n'existe pas pour Git.

**Une branche**, est simplement une référence à un commit existant qui va servir d'origine (de parent), qui sera le point de divergence. La branche sera un ensemble de commits qui aura pour but de corriger un bug ou de développer une fonctionnalité. Ce qui veut dire que le commit référence de la branche évoluera en même temps que le HEAD.
Une branche existera souvent en deux exemplaires dans vos dépôts, une branche locale qui appartient uniquement à votre dépôt local et une branche distante qui appartient à la fois à votre dépôt local et au dépôt distant. Ces deux branches sont bien deux branches distinctes pour votre dépôt git, en revanche git sait qu'il y a un lien entre les deux branches on parle de **tracking de branche ou de branche upstream**.


Une branche peut être **mergé/fusionné** dans une autre branche, ce qui donne lieu à un **commit de merge**, c'est un commit ayant deux parents. 

Dans le cas de branches **fast-forward** on pourra aussi réaliser un rebase plutôt qu'un merge (il s'agit d'une réécriture de l'historique, dans la pratique on fait rarement ça).

![Branches FF](/images/git/git_branche_ff.svg)

Lors d'un merge complexe il est probable de rencontrer des **conflits**, un conflit survient lors d'un merge lorsque des modifications ont eu lieu sur les mêmes lignes d'un même fichier. Il faut alors gérer les conflits, et finaliser le merge ou l'annuler (abort) si c'est trop complexe.
La gestion des modifications au niveau de la ligne dans le fichier est incroyable en ce qui concerne le code, en revanche ça devient problématique sur des fichiers générés comme les projets FME ou sur des binaires, mais ça reste possible.

**Un tag**, au même titre qu'une branche est une référence sur un commit existant, à la différence que la référence n'évoluera pas en même temps que le HEAD.
Un tag peut être annoté ou non (on parle de tag **annoté** ou **léger/lightweight**).

**Le stash** permet de mettre en attente des modifications en cours dans une sorte de cinquième zone de travail. Le stash est uniquement présent sur votre dépôt local, il est lié à un parent mais peut être appliqué sur un autre commit si nécessaire. On peut réaliser trois actions sur un stash : 
- **Apply** : on applique les modifications contenues dans le stash sur notre HEAD, en plus de ce qui est déjà en cours de modification (au risque de prendre un conflit).
- **Delete** : sa suppression du stash et donc potentiellement la perte des modifications si elles ne sont pas appliquées avant.
- **Pop** : Qui est en fait un combo de l'apply suivi du delete.

```{Tip}
  Par retour d'expérience, je préconise de dissocier l'apply et le delete plutôt que d'utiliser pop. Il se peut que dans le cas du pop, la récupération des modifications ne se fasse pas bien et que le delete s'applique quand même auquel cas vos modifications seront partiellement ou totalement perdues.
```
**Le cherry-pick** copie un commit existant pour l'appliquer sur votre HEAD, dans ce cas-là, l'auteur du commit d'origine est aussi l'auteur du commit final, en revanche vous devenez le commiter à la place du commiter d'origine. Le cherry-pick est très intéressant pour déplacer des commits dans l'arborescence en revanche il est à double tranchant, **Cherry-pick un commit sur une branche qui sera mergée avec la branche d'origine entrainera systématiquement un conflit ou une perte de code.**

```{Warning}
  Je passe sous silence les notions très avancées, comme les submodules/subtrees qui ne sont plus utilisées aujourd'hui. Par retour d'expérience, c'est pénible à utiliser, rajoute énormément de complexité sur les tâches simples et n'est pas adapté à nos architectures de développement.
```

**Les hooks** sont des scripts qui peuvent s'exécuter à différents moments (pre-pull, post-pull, pre-push, post-push, ...). avec un hook, il est possible de détecter des changements, d'automatiser des actions sur votre poste, ou autres. Les hooks peuvent être locaux ou distants, sachant que vous ne pourrez pas agir sur un hook distant (ex : système de protection de branche de Gitlab).

### Commandes

Une fiche récapitulative des commandes est disponible à [ce lien](http://documentation-dev.veremes.net/dev/fr/_static/ressources/git/git-cheat-sheet.pdf). 

Dans ces commandes quand je parle de référence, ça veut dire que la commande attend un commit, ça veut dire qu'il prendra une branche, un tag, une référence relative au HEAD, un identifiant sha1 de commit, ...

- **init** : Créer un nouveau dépôt local
- **clone** : Récupère un dépôt distant en local
- **checkout** : Permet de déplacer le HEAD sur une référence (permet notamment de passer en HEAD détaché)
- **add** : Permet de passer des modifications en cours dans la staging area (créer les blobs et les arbres)
- **commit** : créer un nouveau commit avec le contenu de la staging area
- **fetch** : Met à jour les branches distantes sur le dépôt local depuis le dépôt distant
- **merge** : Fusionne deux branches via un commit de merge
- **rebase** : Permet de réécrire l'historique en agissant sur l'arbre, il y a plusieurs types de rebase
- **pull** : réalise un fetch pour mettre à jour toutes les branches distantes du dépôt local depuis le distant, puis enchaine sur un merge via le lien de branche upstream pour mettre à jour votre branche locale.
- **push** : Merge la branche locale dans sa branche distante upstream localement et envoie le résultat sur le serveur distant pour que ce soit disponible pour tout le monde
- **reset** : Permet de déplacer le HEAD de la branche courante sur un autre commit, en conservant les modifications ou non
- **revert** : créer un commit opposé à un commit existant, ce qui aura pour effet d'annuler complètement le commit ciblé (**ATTENTION!** avec les merges).
- **stash** : Permet de créer puis de gérer vos stashs
- **bisect** : Permet de détecter le commit à l'origine d'une erreur

Je ne rentre pas dans le détail des commandes car 99,99999999999999999% du temps vous vous servirez de Git à travers GitKraken.


```{Warning}
  Pour Windows, il est préférable de passer par le Git Bash pour utiliser les commandes Git. L'invite de commande Git Bash s'ouvre directement depuis le menu contextuel en faisant un clique droit sur/dans un dossier.
```

### Bonnes pratiques

- Des commits **"atomiques"** avec un contenu cohérent et fait de manière régulière.
- Des **commentaires** *claires*, *courts* et *soignés* pour la première ligne du commit.
- Ne pas cherry-pick un commit entre deux branches amenées à être mergées.
- Mise à jour **régulière** des branches de travail **avant de merge** dans la branche principale pour éviter les **conflits complexes** à résoudre.
- **Nettoyage régulier** des branches locales et distantes.
- Utilisation du **même nom** pour les branches **locales** et **distantes** liées (la branche distante préfixée par origin/).
- Ne **pousser** sur le dépôt distant que du **code utile** pour le projet, pas de code temporaire.
- Ne **jamais réécrire l'historique** d'une branche qui est déjà poussée sur le dépôt distant.
- Limiter au maximum l'utilisaton du force push.
- Utiliser un workflow de gestion des branches : **GitlabFlow**.
- **Toujours mettre à jour le dépôt local (git pull) avant de pousser les modifications sur le dépôt distant (git push)**.
## GitLab

Il existe plusieurs gestionnaires de dépôts distants pour Git (Github, GitLab, BitBucket, ...).
Nous avons choisi GitLab pour sa similarité avec GitHub, et la possibilité d'en avoir une version on-premise.

```{Warning}
  Gitlab existe aussi en SaaS, au moment de créer votre compte attention de bien vous inscrire sur le [Gitlab de Veremes](https://gitlab.veremes.net/).
  Votre compte devra être validé par administrateur ce qui pourra prendre un peu de temps.
```

### Fonctionnalités

Gitlab est un gestionnaire de dépôt distant basé sur Github à l'origine. Il apporte une interface Web permettant d'interagir avec les dépôts Git et énormément de fonctionnalités additionnels. Comme le suivi de ticket via son système d'issue, l'automatisation de processus via les CI/CD, ou la simplification des revues de code via son système de Merge Request, ou encore simplement l'organisation des dépôts sous une arborescence, ainsi que la gestion très simple de l'accès aux dépôts.

Dans gitlab, **un projet** est un dépôt. Un projet est lié à **un groupe**, qui pourra contenir plusieurs projets, ce qui va permettre d'obtenir une arborescence simplifiant la navigation.

Un projet/groupe peut avoir un accès public ou restreint à certains utilisateurs (qui auront différents niveaux de droits). La quasi-totalité des sous-objets pourront être géré soit sur le projet, soit sur le groupe (droits, labels, fonctionnalités, ...), ce qui dans ce dernier cas impactera tous les projets lui appartenant.

Il y aura cinq niveaux d'accessibilités pour les utilisateurs (et un sixième en comptant l'administration GitLab), vous trouverez le détail des droits de chacun dans la [documentation de GitLab](https://docs.gitlab.com/ee/user/permissions.html).

Gitlab permet de suivre et tracer les bugs/évolutions via un système d'issue, qui pourront être clairement identifié via des **labels** et regrouper dans des **milestone/jalons** ce qui nous sert de roadmap pour nos développements.

![Milestone Gitlab](/images/git/gitlab_milestone.png)

**Les merge request** permettent de gérer le merge directement sur le dépôt distant ou de réaliser un merge classique sur notre poste, ce qui est aussi un gros gain de temps. L'interface permet de faire une revue de code assez rapidement, de gagner en réactivité et améliore le suivi des développements.

![Merge Request Gitlab](/images/git/gitlab_mr.png)

Gitlab intégre également une interface d'édition des fichiers directement sur l'interface Web, avec des fonctionnalités très basiques. 

## GitKraken

Veremes a choisi d'utiliser Gitkraken comme outil graphique pour gérer Git sur nos postes.
GitKraken est un outil très puissant, qui s'adapte aux habitudes de chacun et aux pratiques de l'entreprise. 
Il est très utilisé dans le monde et présente l'avantage d'être compatible sur Windows et Linux.

La documentation de GitKraken est disponible via [ce lien](https://help.gitkraken.com/gitkraken-client/gitkraken-client-home/)

### Installation

Il faut récupérer le package Linux ou l'installeur Windows directement sur le site de [Axosoft](https://www.gitkraken.com/git-client).

Pour linux, il suffit de lancer `sudo dpkg -i mon_paquet_gitkraken.deb`.

```{Warning}
  L'installation via le gestionnaire d'application Ubuntu est fortement déconseillée, les installations snap pouvant poser des problèmes par la suite à cause des droits sur certains dossiers.
```

### Description de l'interface et des fonctionnalités

Il est possible d'avoir des interfaces plus ou moins différentes avec GitKraken, Nus allons déjà voir l'interface de base puis on détaillera une interface que je juge plus pratique. Chacun est libre d'utiliser l'outil comme il le souhaite.

#### Interface de base

![IHM simple](/images/git/gk_ihm_1.png)

Les rectangles rouges délimitent les zones que je vais expliquer par la suite : 
1. Le fils d'Ariane, permet de savoir sur quel dépôt vous êtes, la branche actuellement en cours d'utilisation ou le commit utilisé comme HEAD, et éventuellement le submodules, ... dans certains cas d'utilisation. Cette partie permet également d'alterner entre les dépôts/branches (au prix d'un chargement)
1. Contient la liste des branches locales, c'est-à-dire présente uniquement sur votre poste (ça peut inclure des branches liées à des branches distantes)
1. La liste des branches disponibles sur le serveur distant
1. La liste des tags
1. Boutons permettant les actions principales, pull, fetch, push, gestion des stash
1. Arborescence de commit sur tout le dépôt
1. Ce dernier bloc, vous permet de voir ce qui est en cours de modification dans votre dossier de travail, d'interagir avec votre zone de cache et de commit


Les actions du type suppression/création de branche/tag, revert, reset, cherry-pick, interactive rebase, se fait via le menu contextuel en faisant un clique droit sur un commit ou une référence (branche,tag,...)

![commit menu gk](/images/git/gk_commit_menu.png)

Pour merge/rebase deux branches il faut utiliser le menu en cliquant spécifiquement sur la branche source ou drag & drop la branche source sur la branche destination.

#### Interface mode terminal

J'ai une préférence pour cette interface pour limiter le nombre d'onglets simultanés ouvert dans votre GitKraken.

![IHM simple](/images/git/gk_ihm_2.png)

Pour y accéder il vous suffit d'ouvrir un nouvel onglet et choisir `GitKraken CLI > New Terminal Tab`

![IHM simple](/images/git/gk_open_terminal.png)

En navigant via l'invite de commande, si vous entrez dans un dossier contenant un dépôt Git alors Gitkraken chargera automatiquement les informations de ce dépôt. Ce qui vous permet par exemple de passer d'une application à un de ses modules ou une autre application en faisant vous déplaçant simplement. De plus ce terminal fonctionnant comme un terminal normal vous pouvez lancer d'autre commande comme `code mon_dossier` qui permet d'ouvrir directement Visual Studio Code, ...

En revanche l'interface étant moins complète, il faudra régulièrement utiliser des commandes git très simples comme : 
- git pull 
- git push
- git checkout ma_branche
- git fetch
- git stash
- git branch --all

Par contre l'invite de commande a une auto complétion qui vous aide avec les commandes GitKraken et Git qui est vraiment très utile.

Les interactions avec la zone de gestion des commit ou l'arborescence est identique à l'interface classique.

#### Workspaces

GitKraken permet de regrouper des dépôts et d'en faire un workspace, ce qui vu notre architecture est très pratique. Ces workspaces peuvent être locaux (depuis la version 9.1.0 de GitKraken) ce qui les rend bien plus souples, les workspaces cloud ne peuvent avoir qu'une seule fois le même dépôt, par exemple il faut relocaliser Vitis à chaque changement de workspaces. 

Par défaut le premier onglet de votre GitKraken vous permet de gérer vos workspaces, cliquer ensuite sur le bouton `New Workspace`, une modale s'ouvre.

![GK modal new workspace](/images/git/gk_create_workspace.png)

En cliquant sur `browse repositories` vous pourrez naviguer dans votre système de fichiers pour trouver les dépôts.

```{Tip}
  Pour gagner du temps, naviguer une fois jusqu'à l'application, copier le chemin et valider le premier dépôt. Puis relancez la navigation, coller le chemin de l'application et sélectionnez directement le dossier src, ça importera tous les dépôts de modules en même temps.
```

Vous pourrez ensuite ouvrir le workspace.

![GK wk ex](/images/git/gk_workspace_ex.png)

Le workspace présente plusieurs avantages : 
- Fetch tous les dépôts en même temps
- Visualiser l'état de vos branches courantes pour chaque dépôt en même temps
- Ouvrir un onglet par dépôt facilement
- Ouvrir VS code sur un dépôt ou tous facilement

Le fait d'avoir un workspace par application permet de basculer rapidement d'une application à une autre. Le seul manque à mon sens, est l'intégration avec Gitlab que l'on a avec des workspaces de type Cloud, ce serait vraiment top.

## Autres outils graphiques pour Git

Je ne rentre pas dans le détail de ces outils car leurs utilisations actuellement sont très ponctuelles, mais ce sont des alternatives à GitKraken en cas de manques de licences ou si on a besoin d'un outil plus simple à utiliser.


- [SourceTree](https://www.sourcetreeapp.com/) : Très similaire à GitKraken, il a moins d'intégration avec Gitlab car développé par Atlassian, le propriétaire de BitBucket
- [Github Desktop](https://desktop.github.com/) : Beaucoup plus simpliste, ça fera largement l'affaire pour un utilisateur qui utilise Git rarement ou qui ne fait que des actions très simples (pull, push, commit), pour corriger/écrire de la documentation. 
- [Tortoise Git](https://tortoisegit.org/) : Pour les anciens cramponnés à SVN ou pour avoir un gestionnaire ultra-léger.
- [Git Lens](https://www.gitkraken.com/gitlens) : également développer par Axosoft, c'ets un outil complémentaire à Gitkraken, qui peut le remplacer sur des usages très simplistes, directement dans Visual Studio Code.