# Utilisation d'un système Linux

Dans cette partie on va voir les pièges à éviter et on va essayer de faire un petit tour des paquets trés pratiques dans le cadre du support ou du développement.

Je considère que vous connaissez les commandes de bases (ls, cd, apt, sudo, ...).

## Shell : Un pour tous et tous pour un

Le shell est un programme informatique qui fournit une interface en ligne de commande (CLI) entre l'utilisateur et le système d'exploitation. Le shell est un interpréteur de commandes qui permet à l'utilisateur de saisir des commandes en texte brut pour effectuer des opérations sur le système d'exploitation.

Par abus de langage, on parle de shell ou de bash mais en réalité on ne sait pas toujours ce qu'on manipule et certains de ces interpréteurs ne s'utilisent pas du tout de la même façon et/ou ne prennent pas l'informations de la même source.

Windows va utiliser un Shell spécifique dans ses invites de commande et a sa propre version du shell avec le PowerShell. \
Ubuntu ou Debian vont utiliser un shell amélioré que l'on appelle Bash, mais si je reprend Kali dont j'ai parlé avant qui est une distribution type Debian
par défaut elle utilise zsh à la place de bash.

Sur le système Linux au lancement du shell, les scripts .bashrc seront jouer, il y a un script principal commun a tous les utilisateurs de la machine qu'il est fortement déconseillé de modifier, et un script spécifique à votre utilisateur qui se trouve dans `$HOME/.bashrc`, mais si je reprend zsh, le script s'appelera .zshrc et pourra avoir une syntaxe différente (chose à savoir si vous voulez modifier votre environnement ou créer des alias). 

C'est pour ça qu'au début de beaucoup de script shell on retrouve ce qu'on appelle un shebang qui ressemble à ça : 

```bash
#!/bin/sh

#mon script, ...
cd ..
ls -al
```

La première ligne commenté permet de définir l'interpréteur à utiliser pour le script, qui peut même être python ou php si on ne veut pas faire de shell.

Pour la suite des exemples nous allons toujours parler de bash linux classique.

## Interagir avec les fichiers

Par défaut il y aura toujours le paquet `vi` voir `vim` qui sont deux éditeurs de texte, trés pratiques quand on les connait, mais qui est fortement déconseillé pour des débutants.
Sur un Ubuntu, il y aura généralement `nano` qui est présent de base sur la machine et qui est plus simple d'utilisation mais moins puissant.

Dans tous les cas on aura le plus souvent les commande `cat` et `tail` pour lire des fichiers, ainsi que `awk` et `sed` pour les modifier même si c'est vite laborieux.

## Interroger une base de donnée Postgres

Dans notre cas on va parler de postgres.
En installant le paquet **postgresql**, la machine peut lancer les commandes suivantes (entre autre): 
- `psql` : qui permet de se connecter à une base de donnée et d'exécuter des requêtes SQL
- `pg_dump`: qui permet de faire une sauvegarde de la structure de donnée et des données partiellement ou entiérement
- `pg_dump_all` : qui permet de sauvegarder toutes les bases y compris les rôles mais qui est moins configurable que la précédente
- `pg_restore` : qui permet de jouer un dump pour restaurer une base

## Localiser un fichier 

Le paquet **mlocate** permet de jouer deux commandes : 
- `updatedb` : met à jour une base interne permettant d'avoir une réponse rapide lors de la recherche
- `locate` : Permet de chercher un fichier

Exemple d'utilisation trés pratique, recherche d'un fichier dans un sous dossier avec un `grep` :
```shell
sudo updatedb
sudo locate mon_fichier | grep /var/www/gtf/vas
```

## Récupérer la documentation d'une commande

Pour avoir un descriptif complet d'une commande et de son utilisation il faut utiliser la commande `man` : `man ls` pour la documentaion de ls

Pour avoir une aide simple et ainsi qu'un exemple d'utilisation il suffit le plus souvent d'ajouter l'option `--help` à la commande souhaité

## Analyser le réseau

Chez les clients le plus souvent la base de données se trouvent sur un autre serveur que celui utilisé pour l'applicatif.
Il arrive que pour diverses raisons le réseau bloque la communication entre les deux serveurs, ou un SMTP externe ou un autre serveur.

Dans ce cas là plusieurs outils peuvent vous aider à vérifier si le serveur est accessible et joignable.

### Ping

Le ping est une commande qui permet d'envoyer des paquets d'octets à une machine.
En utilisant une url vous pourrez voir si le DNS résout bien l'adresse IP et si le serveur est joignable.

```{Warning}
  Le ping peut être bloqué par les protections contre le DDOS, donc un retour négatif de la commande ne veut pas dire que le serveur est injoignable. A l'inverse un retour positif assure que le serveur est visible pour la machine qui lance la commande, en revanche ça ne garantie pas que le port de communication soit ouvert.

  C'est pourquoi le ping est rarement suffisant pour s'assurer d'un problème réseau, mais il donne déjà des pistes
```

### Traceroute

Traceroute comme ping va prendre en paramètre une IP ou un domaine. 
Il va premièrement résoudre le domaine en IP dans tous les cas ce qui permettra de tester la résolution DNS, il va ensuite vous afficher un maximum de rebonds par lesquelles passent vos informations.

![traceroute](./images/traceroute.png)

On voit sur cette exemple que je suis bien arrivé dans l'infra d'AWS c'est le dernier rebond, ensuite la commande n'a plus de retour donc on en saura pas plus.

```{Warning}
  Comme pour le ping cette commande est souvent limitée pour des raisons de sécurité. Elle va vous permettre de vérifier la communication jusqu'à un point. Dans l'exemple je suis sûr que j'arrive jusqu'à AWS, donc si ma machine ne reçoit pas ma requête soit j'ai mal configuré mon groupe de sécurité, soit AWS rencontre un problème et je peux m'appuyer sur le modèle de responsabilité partagé pour que le support agisse sur l'infrastructure.
```

### Nmap

nmap bien utilisé peut vous permettre de reproduire toute une architecture réseau ainsi que de détecter les ports ouverts.

![nmap](./images/nmap.png)

Sur l'exemple on voit une liste de port ouvert et fermé.
Le port 22 pour SSH, 80/443 pour le HTTP, ...

A ce moment là vous avez une idée clair de l'accéssibilité de la machine distante et en cas d'erreur vous pouvez écarter ou incriminer le réseau.

```{Warning}
  nmap ne sera jamais installé sur les machines par défaut, il faut toujours demander au client avant d'installer une commande de ce type et demander l'autorisation de scanner la machine cible.

  Nmap est une commande connue et est souvent détecté comme une tentative d'intrusion vu qu'un attaquant va réguliérmeent l'utiliser comme première commande avant d'attaquer réellement la machine.
```

### Telnet

Quand on en arrive là, généralement c'est que ça devient compliqué.

Telnet vous permet de contacter un serveur sur un port et de lui envoyer des trames de données.

Une utilisation que j'ai assez souvent faite c'est pour débugguer les SMTP. Avec telnet on peut contacter le serveur sur le port, puis vérfiier sa réponse et enfin tester l'authentification, voir même l'envoi de mail complet.

On peut aussi l'utiliser pour tester de l'IMAP ou tout autre protocole.

C'est une commande assez complexe dont l'utilisation dépend de ce que l'on veut tester, je vous conseille de chercher l'utilisation du protocole à tester avec telnet directement pour votre cas spécifiquement.

## Récupération des statistiques de la machine

Par défaut les commandes `top` et `ps -ax` devraient toujours être présente et fonctionner.
Une autre commande à installer qui est souvent pratique est `htop`, qui est plus simple à analyser, dynamique et présente d'autre fonctionnalités comme le `kill` via la sélection de processus ou la recherche de processus.

![htop](./images/htop.png)

Deux autres commande qui seront trés pratique pour récupérer l'état des disques sont `df` et `du`.

`df` donnera une vue globale de l'état de la machine, alors que `du` permettra de voir létat d'un dossier sur le disque

![df_du](./images/df.png)

## Utiliser la commande précédente

En shell il est possible d'utiliser `!!` pour rejouer la commande précédente.

Exemple d'utilisation : 
```shell
rm -rf mondossier
# Vous n'avez pas les droits pour ...
sudo !!
# Equivaut à sudo rm -rf mondossier
ls
# Mais je voulais la version liste complète
!! -al
```

## Se déplacer rapidement dans l'arborescence

la commande `cd` est basique et vous la connaissez surement, par contre il y a quelques astuces pour gagner du temps : 
- `cd` sans paramètre ramène dans le dossier $HOME, ça équivaut à faire `cd ~`
- `cd -` ramène à l'emplacement précédent

## Chercher une commande dans l'historique

La commande `history` retrace toutes les commandes jouées sur la machine sous certaines conditions.

Si vous avez plusieurs terminaux ouvert il récupère pas tout, en SSH c'est relatif à votre utilisateur et ça prend pas tout. Bref ce n'est pas parfait, mais ça peut vous permettre de retrouver une commande que vous voulez rejouer ou de voir ce qui s'est passé sur la machine.

Exemple : Je ne retrouve plus la commande pour compiler la documentation `history | grep sphinx`.

## Usurper www-data

Sur un serveur des clients, pour s'éviter des problème de droits quand on veut lancer les moteurs par exemple, c'est plus pratique de le faire avec `www-data`, mais comme c'est l'utilisateur d'apache c'est pas si simple. Mais la commande `su` permet de le faire.

```shell
sudo su - www-data -s /bin/bash 
```

## Chainer des commandes

Vous pouvez chainer des commandes en utilisant `&&` ou en les séparant par des `;` : 

```shell
# Permet de lister un sous-dossier et de revenir à la position précédente
cd sous-dossier/ && ls && cd ..
cd sous-dossier/; ls; cd ..
```

La syntaxe `&&` lancera les commandes les unes aprés les autres et ne jouera la commande suivante que si la commande en cours se termine en succés. Alors que la syntaxe utilisant `;` chainera les commandes sans tenir compte de l'état de la commande précédente.

## Utiliser les sorties d'une commande

Un exécutable qui affiche un résultat dans le terminal envoi du texte dans la sortie standard `stdout` ou dans la sortie d'erreur `stderr`.

Il est possible de récupérer cette sortie pour la rediriger vers l'entrée standard `stdin` d'une autre commande en utilisant `|`, l'usage le plus courant est d'envoyer ça dans un `grep` pour chercher une chaine de caractère.

```shell
# recherche dans un fichier texte
cat mon_fichier.txt | grep "Ma chaine dans le fichier"
# Exemple pour zipper certains fichier dans un sous-dossier 
find . | egrep "2023-05|2023_05" | sudo zip -@ logs_2023-05.zip
```

Dans cette exemple, la commande `cat` lit un fichier et envoi son contenu dans stdout, avec le pipe on redirige stdout dans le stdin de la commande grep pour rechercher une chaine de caractère dans le fichier.

Il est également possible de rediriger `stdout`et `stderr`, vers un fichier ou l'un vers l'autre en utilisant `>` ou `>>`.

```shell
# écrit le stdout de la commande vers le fichier result.txt (le fichier sera vidé à chaque fois)
ls -al > result.txt
# écrit le stdout de la commande vers le fichier result.txt (le fichier ne sera pas vidé et on ajoutera à la fin)
ls -al >> result.txt
# Redirige stderr (descripteur 2) vers stdout (descripteur 1) lui même renvoyer dans le fichier txt
ls -al 2>&1 >> return.txt
# Isolation des erreur dans des fichiers différents
ls -al >return.txt 2>error.txt
```
